@@ -66,6 +66,25 @@ class is_enrolled_in_ccdf(Variable):
     label = u"CCDF enrollment status"
 
 
+class ccdf_income(Variable):
+    value_type = float
+    entity = SPMUnit
+    label = u"Income"
+    definition_period = YEAR
+
+
+class ccdf_income_to_smi_ratio(Variable):
+    value_type = float
+    entity = SPMUnit
+    label = u"Income to SMI ratio"
+    definition_period = YEAR
+
+    def formula(spm_unit, period, parameters):
+        income = spm_unit("ccdf_income", period)
+        smi = spm_unit("hhs_smi", period)
+        return income / smi
+
+
 class is_ccdf_initial_income_eligible(Variable):
     value_type = bool
     entity = SPMUnit
