@@ -1,6 +1,4 @@
-from openfisca_core.model_api import *
-from openfisca_us.entities import *
-from openfisca_us.tools.general import *
+from openfisca_us.model_api import *
 
 
 class net_income(Variable):
