@@ -648,7 +648,8 @@ class e07300(Variable):
 class filer_e07300(Variable):
     value_type = float
     entity = TaxUnit
-    label = u"Foreign tax credit (Form 1116) for the tax unit (excluding dependents)"
+    label = u"Foreign tax credit (Form 1114) for the tax unit (excluding dependents)"
+    unit = "currency-USD"
     definition_period = YEAR
 
     def formula(tax_unit, period, parameters):
@@ -842,6 +843,17 @@ class e24515(Variable):
     documentation = """Sch D: Un-Recaptured Section 1250 Gain"""
 
 
+class filer_e24515(Variable):
+    value_type = float
+    entity = TaxUnit
+    label = u"Section 1250 Gain (un-recaptured) for the tax unit (excluding dependents)"
+    unit = "currency-USD"
+    definition_period = YEAR
+
+    def formula(tax_unit, period, parameters):
+        return tax_unit_non_dep_sum("e24515", tax_unit, period)
+
+
 class e24518(Variable):
     value_type = float
     entity = Person
@@ -849,6 +861,16 @@ class e24518(Variable):
     documentation = """Sch D: 28% Rate Gain or Loss"""
 
 
+class filer_e24518(Variable):
+    value_type = float
+    entity = TaxUnit
+    label = u"Sch D Rate Gain or Loss for the tax unit (excluding dependents)"
+    definition_period = YEAR
+
+    def formula(tax_unit, period, parameters):
+        return tax_unit_non_dep_sum("e24518", tax_unit, period)
+
+
 class e26270(Variable):
     value_type = float
     entity = Person
@@ -909,6 +931,16 @@ class e58990(Variable):
     documentation = """Investment income elected amount from Form 4952"""
 
 
+class filer_e58990(Variable):
+    value_type = float
+    entity = TaxUnit
+    label = u"Investment income (Form 3952) for the tax unit (excluding dependents)"
+    definition_period = YEAR
+
+    def formula(tax_unit, period, parameters):
+        return tax_unit_non_dep_sum("e58990", tax_unit, period)
+
+
 class e62900(Variable):
     value_type = float
     entity = Person
@@ -918,15 +950,32 @@ class e62900(Variable):
     )
 
 
+class filer_e62900(Variable):
+    value_type = float
+    entity = TaxUnit
+    label = u"AMT foreign tax credit (Form 6251) for the tax unit (excluding dependents)"
+    unit = "currency-USD"
+    definition_period = YEAR
+
+    def formula(tax_unit, period, parameters):
+        return tax_unit_non_dep_sum("e62900", tax_unit, period)
+
+
 class e87530(Variable):
     value_type = float
     entity = Person
     definition_period = YEAR
+    label = "Qualified tuition expenses"
     documentation = (
-        """Adjusted qualified lifetime learning expenses for all students"""
+        "Adjusted qualified lifetime learning expenses for all students"
     )
 
 
+qualified_tuition_expenses = variable_alias(
+    "qualified_tuition_expenses", e87530
+)
+
+
 class elderly_dependents(Variable):
     value_type = int
     entity = TaxUnit
